;-----------------------------------------------------------------------------
;TITLE "- GPWS 98 V1.8 MODULE"
;DESCRIPTION "Ground Proximity Warning System 98 V1.8 "
;DESCRIPTION "(C) 1998 Wilco van Deijl "
;-----------------------------------------------------------------------------

;Compile with APLC32 V1.24 or higher, differences between versions which need
;to be compiled with the -a option and without this option are indicated by:
;[m]   ->  compile without -a option  (altitudes in metres)
;[-a]  ->  compile with the -a option (altitudes in ft, alt compensation off)
;Differences between standard and module version are indicated by {} comments
;throughout this code. {also disable title and description for module}

; Startup ********************************************************************
APL_SOUND=1
DECLARE init                                ;{add for module}
IF init                                     ;{add for module} 
    GOTO gpws                               ;{add for module} 
ENDIF                                       ;{add for module} 
init=1                                      ;{add for module} 
;ADV_KEYS CLEAR                              ;{delete for module}
;ONEXP_CLEAR                                 ;{delete for module}

; Declaring variables ********************************************************
DECLARE DH_key, GPWS_key, BANK_key, TRIM_key, END_key
DECLARE altcall, aplrate, bank, bankangle, com1rate, dh, gear
DECLARE gpws, gpwsmode, ias, ias0, lastcall, ra, ra0, sinkalt
DECLARE tdtime, time, time0, towarn, val, voice, vs, x1, x2
DECLARE alertalt, altalert, altdev, trim, trimsound
DECLARE aplc32_version, aplrate_min, perf_high

; Testing systems / apl performance ************************************************************
PRINT_TIME 20
;PRINT "TESTING SYSTEMS"                     ;{delete for module}
IF aplc32_version>0 AND aplc32_version<124
    PRINT "You need to upgrade your copy of APLC32"
    WAIT 8
ENDIF
aplrate_min=13                              ;minimum apl rate
perf_high=5                                 ;perf start value
GOSUB set_aplrate                           ;set apl rate (by adjusting perf)

; Default settings / Initialising variables **********************************
com1rate=9              ;Default rate of ATC (0-9)         {add for module}
gpws=5                  ;GPWS mode     (0=off, 1=altitude calls only, 5=on)
bankangle=35            ;Bank angle    (0=off, 35, 50, 65, ... degrees)
trimsound=1             ;Trim sound    (0=off, 1=manual, 2=on)
towarn=1                ;Take-off warn (0=off, 1=on) 
IF ENGINE_TYPE=1        ;Default settings for jet aircraft:
    gear=6              ;RA correction for gear height (overridden on ground)
ELSE                    ;Default settings for other aircraft:
    gear=4              ;RA correction for gear height (overridden on ground)
ENDIF                   ;
trim=ELEVATOR_TRIM_POSITION

; Assigning keys *************************************************************
DH_key = KV(SHIFT "d")       ;You can change these keys
GPWS_key = KV(SHIFT "g")     
BANK_key = KV(SHIFT "b")     
TRIM_key = KV(SHIFT "t")     
END_key = KV(CTRL "u")       

; Adding keys **************************************************************** 
ADV_KEYS ADD, DH_key, GPWS_key, BANK_key, TRIM_key, END_key
ONKEY DH_key GOSUB set_dh                   ;{delete for ProFlight98 module}
ONKEY GPWS_key GOSUB set_gpws               ;{delete for ProFlight98 module}
ONKEY BANK_key GOSUB set_bankangle          ;{delete for ProFlight98 module}
ONKEY TRIM_key GOSUB set_trimsound          ;{delete for ProFlight98 module}
ONKEY END_key GOSUB gpws_end                ;{delete for ProFlight98 module}
							   
; Event monitors *************************************************************
PRINT_TIME 4
;GOSUB hal9000                               ;{delete for module}
PLAY "-ss1"
ONEXP_CHANGE AUTOPILOT_MASTER GOSUB ap_disc

; Common procedures **********************************************************
DEFINE PROCEDURE on_gnd()                             ;RA correction
    IF INRANGE(PLANE_ALT-GROUND_ALTITUDE,0,6) AND ON_GROUND             ;[m] 
        gear=MTOF(PLANE_ALT-GROUND_ALTITUDE)                            ;[m]
;    IF INRANGE(PLANE_ALT-GROUND_ALTITUDE,0,20) AND ON_GROUND            ;[-a]
;        gear=PLANE_ALT-GROUND_ALTITUDE                                  ;[-a]
    ENDIF
ENDDEFINE

DEFINE PROCEDURE ra()                                 ;Radio Altitude
    ra=MTOF(PLANE_ALT-GROUND_ALTITUDE)-gear                             ;[m]
;    ra=PLANE_ALT-GROUND_ALTITUDE-gear                                   ;[-a]
ENDDEFINE

DEFINE PROCEDURE timer(start_time)
    time=60*MINUTE+SECOND-start_time
    IF time<0                                   
	time=time+3600
    ENDIF
ENDDEFINE

DEFINE FUNCTION timer_start()
    RETURN 60*MINUTE+SECOND
ENDDEFINE

;-----------------------------------------------------------------------------
; GPWS main program
;-----------------------------------------------------------------------------
gpws:
    ias0=ias
    ias=VELOCITY_INDICATED                            ;Indicated Airspeed
    ra()                                              ;Radio Altitude
    IF ON_GROUND
        timer(tdtime)                                 ;touchdown timer
        IF THROTTLE_1>.7 AND time>9                   ;go around: no warn/call
            IF (NOT INRANGE(ELEVATOR_TRIM_POSITION,-8000,8000) OR NOT INRANGE(FLAPS_LEFT_POSITION,3300,22000) OR PARKING_BRAKES) AND towarn>0  ;[m]
;            IF (NOT INRANGE(ELEVATOR_TRIM_POSITION,-.5,.5) OR NOT INRANGE(FLAPS_LEFT_POSITION,.1,.67) OR PARKING_BRAKES) AND towarn>0          ;[-a]
                PLAY ""
                PLAY "-towarn"                        ;take-off config warning
                WAIT 1.6
                GOTO gpws
            ENDIF
        ENDIF
        on_gnd()
    ELSEIF INRANGE(ra,30,2450) 
        IF ias0-ias>9 AND INRANGE(ias,1,180) AND ra<1500 AND PLANE_PITCH<25 
            x1=(WIND_SURF_VEL>10 AND INRANGE(MTOF(WIND_SURF_DEPTH)-ra,-200,200)) OR (WIND_LOW_VEL>10 AND (INRANGE(MTOF(WIND_LOW_BASE)-ra,-200,200) OR INRANGE(MTOF(WIND_LOW_TOP)-ra,-200,200)))                                               ;[m]
            x1=x1 OR (WIND_MID_VEL>10 AND (INRANGE(MTOF(WIND_MID_BASE)-ra,-200,200) OR INRANGE(MTOF(WIND_MID_TOP)-ra,-200,200))) OR (WIND_UP_VEL>10 AND (INRANGE(MTOF(WIND_UP_BASE)-ra,-200,200) OR INRANGE(MTOF(WIND_UP_TOP)-ra,-200,200)))  ;[m]
;            x1=(WIND_SURF_VEL>10 AND INRANGE(WIND_SURF_DEPTH-ra,-200,200)) OR (WIND_LOW_VEL>10 AND (INRANGE(WIND_LOW_BASE-ra,-200,200) OR INRANGE(WIND_LOW_TOP-ra,-200,200)))                                                                 ;[-a]
;            x1=x1 OR (WIND_MID_VEL>10 AND (INRANGE(WIND_MID_BASE-ra,-200,200) OR INRANGE(WIND_MID_TOP-ra,-200,200))) OR (WIND_UP_VEL>10 AND (INRANGE(WIND_UP_BASE-ra,-200,200) OR INRANGE(WIND_UP_TOP-ra,-200,200)))                          ;[-a]
            IF x1 AND gpws>1              
                PLAY ""
                PLAY "-windshr"                       ;mode7 windshear
                WAIT 3
                ias=VELOCITY_INDICATED
                GOTO gpws
            ENDIF
        ENDIF
        vs=VELOCITY_Y*196.8                           ;vert spd m/s     ;[m]
;        vs=VELOCITY_Y                                 ;vert spd ft/min  ;[-a]
        IF vs<100 
            IF ra<1000 
                IF vs<-1.2*ra-1000 AND gpws>1         
                    IF vs<-1.2*ra-2000
                        GOTO gpws_terrain             ;mode2
                    ELSEIF vs<-1.8*ra-1400
                        GOTO gpws_pullup              ;mode1
                    ELSE
                        GOTO gpws_sinkrate            ;mode1
                    ENDIF                      
                ELSEIF GEAR_2_POSITION=0 AND (ra<500 OR ias>.12*ra+130) AND sinkalt=0 AND gpws>2
                    GOSUB altcall_off                 ;alt call monitor off
                    gpwsmode=4                        ;mode4a gear
                    PLAY ""                           ;cut off sounds
                    IF ias>190
                        PLAY "-toolowt"               
                    ELSE
                        PLAY "-gear"                  
                    ENDIF
                    WAIT 2
                    GOTO gpws
                ELSEIF FLAPS_LEFT_POSITION<21500 AND (ra<245 OR ias>.12*ra+130) AND sinkalt=0 AND gpws>3  ;[m]
;                ELSEIF FLAPS_LEFT_POSITION<.66 AND (ra<245 OR ias>.12*ra+130) AND sinkalt=0 AND gpws>3    ;[-a]
                    GOSUB altcall_off             
                    gpwsmode=4                        ;mode4b flaps <20
                    PLAY ""                       
                    IF ias>159
                        PLAY "-toolowt"               
                    ELSE
                        PLAY "-flaps"                 
                    ENDIF
                    WAIT 2
                    GOTO gpws
                ELSEIF VOR_1_GS_AVAIL AND VOR_1_GS_DEVIATION>.65 AND (ra>150 OR VOR_1_GS_DEVIATION>.65+(150-ra)/170) AND (ra<500 OR INRANGE(VOR_1_COURSE_DEVIATION,-.999,.999)) AND gpws>4
                    IF gpwsmode!=5                    ;<150ft less sensitive
                        gpwsmode=5                    ;mode5 glide slope
                        PLAY ""                       
                    ENDIF
                    PLAY "-gs"                        
                    WAIT .003*ra/VOR_1_GS_DEVIATION+.8
                    GOTO gpws  
                ENDIF
            ELSEIF vs<-1.9*ra-300 AND gpws>1          ;RA >1000ft
                IF vs<-4.5*ra+1300                
                    GOTO gpws_terrain                 ;mode2
                ELSEIF vs<-2.6*ra-600
                    GOTO gpws_pullup                  ;mode1
                ELSE
                    GOTO gpws_sinkrate                ;mode1
                ENDIF                      
            ENDIF
        ELSEIF ra<245 AND (GEAR_2_POSITION=0 OR FLAPS_LEFT_POSITION<21500)  ;[m]
;        ELSEIF ra<245 AND (GEAR_2_POSITION=0 OR FLAPS_LEFT_POSITION<.66)    ;[-a]
            sinkalt=1             ;mode3 arm (take-off / go-around <245ft
        ENDIF                     ;           with gear or flaps raised)
        IF sinkalt>0
            IF (ra>700 AND ias<190) OR ra>1333 OR (GEAR_2_POSITION>0 AND FLAPS_LEFT_POSITION>22000)  ;[m]
;            IF (ra>700 AND ias<190) OR ra>1333 OR (GEAR_2_POSITION>0 AND FLAPS_LEFT_POSITION>.67)    ;[-a]
                sinkalt=0                             ;mode3 disarm
            ELSEIF PLANE_ALT>sinkalt 
                sinkalt=PLANE_ALT
            ELSEIF sinkalt-PLANE_ALT>ra/33 AND gpws>1                   ;[m]
;            ELSEIF sinkalt-PLANE_ALT>ra/10 AND gpws>1                   ;[-a]
                IF gpwsmode!=3                        
                    gpwsmode=3                        ;mode3 don't sink
                    PLAY ""                       
                ENDIF
                PLAY "-dontsnk"                       
                WAIT 1.5
                GOTO gpws  
            ENDIF
        ENDIF
    ENDIF                    
    gpwsmode=0                                        ;reset warnings
    ra()
    IF ra>lastcall*1.03+20 AND VELOCITY_Y<0
        GOSUB altcall_off                             ;reset mode6 alt calls
        GOSUB altcall_on  
    ENDIF
    IF NOT INRANGE(PLANE_BANK,-10,10) AND (NOT INRANGE(PLANE_BANK,-bankangle,bankangle) OR NOT INRANGE(PLANE_BANK,-.25*ra-2.5,.25*ra+2.5)) AND bankangle>0 AND gpws>1
        x1=PLANE_BANK                                 
        IF x1<0
            x1=-x1
        ENDIF
        IF x1>bank OR x1>bankangle+10 OR (ra<130 AND x1>.36*ra+3.6)
            PLAY ""
            PLAY "-bnkangl"                           ;mode6 bank angle
            IF ra>130
                WAIT 1.5                              ;<130ft no pause
            ENDIF
            PLAY "-bnkangl"
            WAIT 1.5
            IF bank<x1*1.14
                bank=x1*1.14                          ;increase limit
            ENDIF
        ENDIF
    ELSEIF bank>0
        bank=0                                        ;reset mode6 bank angle
    ENDIF                    
    GOSUB gpws_input         ;{add for module}        ;backup key input
    GOSUB alt_alert                                   ;altitude alert
    GOSUB trim                                        ;trim
    aplrate=aplrate+1                       ;{add for module}
    IF aplrate<10-com1rate OR com1rate=0    ;{add for module}   
        GOTO gpws       
    ENDIF                                   ;{add for module}
    aplrate=0                               ;{add for module}
    RETURN                                  ;{add for module}

;-----------------------------------------------------------------------------
; GPWS subroutines
;-----------------------------------------------------------------------------
gpws_end:
    PLAY ""
    PLAY "-ss1"
    ADV_KEYS CLEAR
    END

gpws_input:
    trim=ELEVATOR_TRIM_POSITION
gpws_inkey:                                           
    WAIT 0                   
    val=KEY                 
    IF val=0
        RETURN
    ENDIF
gpws_inkey2:                                          ;backup key input
    IF val=DH_key             
        GOSUB set_dh         
    ELSEIF val=GPWS_key      
        GOSUB set_gpws
    ELSEIF val=BANK_key
        GOSUB set_bankangle
    ELSEIF val=TRIM_key
        GOSUB set_trimsound
    ELSEIF val=END_key
        GOTO gpws_end
    ELSE
        trim=ELEVATOR_TRIM_POSITION
        RETURN
    ENDIF
    WAIT 0,KEYPRESS
    WAIT 0,KEYPRESS
    WAIT 2,KEYPRESS
    val=KEY                 
    GOTO gpws_inkey2

gpws_pullup:                                          
    IF gpwsmode=1
        GOSUB altcall_off                             
        PLAY "-pullup"                                ;mode1 pull up
        WAIT 1.2
        GOTO gpws
    ENDIF
gpws_sinkrate:                                        
    IF gpwsmode!=1
        gpwsmode=1                                    ;mode1 sink rate
        PLAY ""                                       
    ENDIF
    PLAY "-snkrate"
    WAIT 1.5
    GOTO gpws  

gpws_terrain:                                         
    IF gpwsmode!=2 
        GOSUB altcall_off                             
        PLAY ""                                       
        gpwsmode=2                                    ;mode2a+b terrain
        PLAY "-terrain"                               
        WAIT 1.2
        PLAY "-terrain"
    ELSEIF GEAR_2_POSITION>0 AND FLAPS_LEFT_POSITION>.67
        PLAY "-terrain"                               ;mode2a+b terrain
    ELSE
        PLAY "-pullup"                                ;mode2a pull up
    ENDIF
    WAIT 1.2
    GOTO gpws 

;-----------------------------------------------------------------------------
; GPWS test / APL rate
;-----------------------------------------------------------------------------
hal9000:
    IF RANDOM<.8 OR HOUR<3
        PRINT "GPWS OK"
    ELSEIF HOUR<10
        PRINT "GPWS OK, GOOD MORNING"
    ELSEIF HOUR<18
        PRINT "GPWS OK, GOOD DAY"
    ELSE
        PRINT "GPWS OK, GOOD EVENING"
    ENDIF
    RETURN

set_aplrate:
    PRECISION 3,0,0
    GOSUB test_aplrate                                ;reset timer
    GOSUB test_aplrate                                ;test apl rate
    x1=INT(perf_high*aplrate_min/aplrate)
    IF NOT INRANGE(x1-perf_high,-2,2)
        perf_high=x1
        GOTO set_aplrate
    ENDIF
    WHILE aplrate>aplrate_min
        perf_high=perf_high-1
        GOSUB test_aplrate
    ENDWHILE
    WHILE aplrate<aplrate_min
        perf_high=perf_high+1
        GOSUB test_aplrate
    ENDWHILE
    RETURN

test_aplrate:
    IF perf_high>0
        PERFORMANCE HIGH perf_high
    ELSEIF perf_high=0
        PERFORMANCE MEDIUM 1
    ELSEIF perf_high=-1
        PERFORMANCE LOW 1
    ELSE
        PRINT "!SYSTEM FAILURE ",aplrate_min,aplrate,perf_high," - RESTART FS98"
        PLAY "-ss2"
        WAIT 20
        END
    ENDIF
    aplrate=0
    time=0
    x1=FUNCTION timer_start()
    WHILE time<1
        aplrate=aplrate+1
        timer(x1)  
    ENDWHILE
;    PRINT "TESTING SYSTEMS ",aplrate_min,aplrate,perf_high  ;{delete for module}
    RETURN

;-----------------------------------------------------------------------------
; Altitude calls
;-----------------------------------------------------------------------------
altcall_off:                                          ;mode6 alt monitor off
    IF lastcall<20
        RETURN
    ELSEIF lastcall<50
        ONEXP_CLEAR call10
        ONEXP_CLEAR call20
        ONEXP_CLEAR call30
    ELSEIF lastcall<200
        ONEXP_CLEAR call40
        ONEXP_CLEAR call50
        ONEXP_CLEAR call100
    ELSEIF lastcall<580
        ONEXP_CLEAR call130
        ONEXP_CLEAR call200
        ONEXP_CLEAR call300
        ONEXP_CLEAR call400
    ELSE
        ONEXP_CLEAR call500
        ONEXP_CLEAR call580
        ONEXP_CLEAR call1000
        ONEXP_CLEAR call2500
    ENDIF 
    lastcall=-99
    ONEXP_TRUE ON_GROUND GOSUB call0
    RETURN

altcall_on:                                           ;mode6 set alt monitor
    IF gpws=0                                         ;GPWS OFF: no alt calls
        lastcall=60000
        RETURN
    ENDIF
    ONEXP_CLEAR call0
    IF ra<50
        ON ra/10 GOSUB alt0, alt10, alt20, alt30, alt40
    ELSEIF ra<580
        ON ra/100 GOSUB alt50, alt100, alt200, alt300, alt400, alt500
    ELSEIF ra<1000
        GOSUB alt580
    ELSEIF ra<2500
        GOSUB alt1000
    ELSE
        GOSUB alt2500
    ENDIF
    RETURN

alt2500:
    lastcall=60000
    altcall=FTOM(2510+gear)                                             ;[m]
;    altcall=2510+gear                                                   ;[-a]
    ONEXP_TRUE PLANE_ALT-GROUND_ALTITUDE<altcall GOSUB call2500
    RETURN
call2500:
    IF PLANE_ALT-GROUND_ALTITUDE>altcall-20
        APL_SOUND=1
        PLAY ""              ;{delete for module}
        PLAY "-2500"
        WAIT 1.3
    ENDIF
    ONEXP_CLEAR call2500
alt1000:
    lastcall=2500
    altcall=FTOM(1010+gear)                                             ;[m]
;    altcall=1010+gear                                                   ;[-a]
    ONEXP_TRUE PLANE_ALT-GROUND_ALTITUDE<altcall GOSUB call1000
    RETURN
call1000:
    IF PLANE_ALT-GROUND_ALTITUDE>altcall-20
        APL_SOUND=1
        PLAY ""              ;{delete for module}
        PLAY "-1000"
        WAIT .9
    ENDIF
    ONEXP_CLEAR call1000
alt580:
    lastcall=1000
    altcall=FTOM(585+gear)                                              ;[m]
;    altcall=585+gear                                                    ;[-a]
    ONEXP_TRUE PLANE_ALT-GROUND_ALTITUDE<altcall GOSUB call580
    RETURN
call580:
    IF dh=500 AND PLANE_ALT-GROUND_ALTITUDE>altcall-20
        APL_SOUND=1
        PLAY ""
        PLAY "-apprmin"
    ENDIF
    ONEXP_CLEAR call580
alt500:
    lastcall=580
    altcall=FTOM(505+gear)                                              ;[m]
;    altcall=505+gear                                                    ;[-a]
    ONEXP_TRUE PLANE_ALT-GROUND_ALTITUDE<altcall GOSUB call500
    RETURN
call500:
    IF PLANE_ALT-GROUND_ALTITUDE>altcall-20
        APL_SOUND=1
        PLAY ""
        IF dh=500
            PLAY "-min"
        ELSE
            PLAY "-500"          
            IF dh=400
                PLAY "-apprmin"
            ENDIF
        ENDIF 
    ENDIF
    ONEXP_CLEAR call500
alt400:
    lastcall=500
    altcall=FTOM(405+gear)                                              ;[m]
;    altcall=405+gear                                                    ;[-a]
    ONEXP_TRUE PLANE_ALT-GROUND_ALTITUDE<altcall GOSUB call400
    RETURN
call400:
    IF PLANE_ALT-GROUND_ALTITUDE>altcall-20
        APL_SOUND=1
        PLAY ""
        IF dh=400
            PLAY "-min"
        ELSE
            PLAY "-400"          
            IF dh=300
                PLAY "-apprmin"
            ENDIF
        ENDIF 
    ENDIF
    ONEXP_CLEAR call400
alt300:
    lastcall=400
    altcall=FTOM(305+gear)                                              ;[m]
;    altcall=305+gear                                                    ;[-a]
    ONEXP_TRUE PLANE_ALT-GROUND_ALTITUDE<altcall GOSUB call300
    RETURN
call300:
    IF PLANE_ALT-GROUND_ALTITUDE>altcall-20
        APL_SOUND=1
        PLAY ""
        IF dh=300
            PLAY "-min"
        ELSE
            PLAY "-300"          
            IF dh=200
                PLAY "-apprmin"
            ENDIF
        ENDIF 
    ENDIF
    ONEXP_CLEAR call300
alt200:
    lastcall=300
    altcall=FTOM(205+gear)                                              ;[m]
;    altcall=205+gear                                                    ;[-a]
    ONEXP_TRUE PLANE_ALT-GROUND_ALTITUDE<altcall GOSUB call200
    RETURN
call200:
    IF PLANE_ALT-GROUND_ALTITUDE>altcall-20
        APL_SOUND=1
        PLAY ""
        IF dh=200
            PLAY "-min"
        ELSE
            PLAY "-200"          
            IF dh=100
                PLAY "-apprmin"
            ENDIF
        ENDIF 
    ENDIF
    ONEXP_CLEAR call200
alt130:
    lastcall=200
    altcall=FTOM(135+gear)                                              ;[m]
;    altcall=135+gear                                                    ;[-a]
    ONEXP_TRUE PLANE_ALT-GROUND_ALTITUDE<altcall GOSUB call130
    RETURN
call130:
    IF PLANE_ALT-GROUND_ALTITUDE>altcall-20
        IF dh=50
            PLAY "-apprmin"
        ENDIF
    ENDIF
    ONEXP_CLEAR call130
alt100:
    lastcall=130
    altcall=FTOM(105+gear)                                              ;[m]
;    altcall=105+gear                                                    ;[-a]
    ONEXP_TRUE PLANE_ALT-GROUND_ALTITUDE<altcall GOSUB call100
    RETURN
call100:
    IF PLANE_ALT-GROUND_ALTITUDE>altcall-20
        APL_SOUND=1
        PLAY ""
        IF dh=100
            PLAY "-min"
        ELSE
            PLAY "-100"          
            IF dh=20
                PLAY "-apprmin"
            ENDIF
        ENDIF 
    ENDIF
    ONEXP_CLEAR call100
alt50:
    lastcall=100
    altcall=FTOM(55+gear)                                               ;[m]
;    altcall=55+gear                                                     ;[-a]
    ONEXP_TRUE PLANE_ALT-GROUND_ALTITUDE<altcall GOSUB call50
    RETURN
call50:
    APL_SOUND=1
    PLAY ""
    IF dh=50
        PLAY "-min"
    ELSE
        PLAY "-50"          
    ENDIF 
    ONEXP_CLEAR call50
alt40:
    lastcall=50
    altcall=FTOM(45+gear)                                               ;[m]
;    altcall=45+gear                                                     ;[-a]
    ONEXP_TRUE PLANE_ALT-GROUND_ALTITUDE<altcall GOSUB call40
    RETURN
call40:
    PLAY ""
    PLAY "-40"          
    ONEXP_CLEAR call40
alt30:
    lastcall=40
    altcall=FTOM(35+gear)                                               ;[m]
;    altcall=35+gear                                                     ;[-a]
    ONEXP_TRUE PLANE_ALT-GROUND_ALTITUDE<altcall GOSUB call30
    RETURN
call30:
    PLAY ""
    PLAY "-30"          
    ONEXP_CLEAR call30
alt20:
    lastcall=30
    altcall=FTOM(25+gear)                                               ;[m]
;    altcall=25+gear                                                     ;[-a]
    ONEXP_TRUE PLANE_ALT-GROUND_ALTITUDE<altcall GOSUB call20
    RETURN
call20:
    PLAY ""
    IF dh=20
        PLAY "-min"
    ELSE
        PLAY "-20"          
    ENDIF 
    ONEXP_CLEAR call20
alt10:
    lastcall=20
    altcall=FTOM(15+gear)                                               ;[m]
;    altcall=15+gear                                                     ;[-a]
    ONEXP_TRUE PLANE_ALT-GROUND_ALTITUDE<altcall GOSUB call10
    RETURN
call10:
    PLAY ""
    PLAY "-10"          
    ONEXP_CLEAR call10
alt0:
    lastcall=10
    ONEXP_TRUE ON_GROUND GOSUB call0
    RETURN
call0:                                                ;touchdown
    ONEXP_CLEAR call0
    IF gpws>1                                         ;reset g/s-inhibit
        gpws=5                                        ;      flap-inhibit
    ENDIF                                             ;      gear-inhibit
    lastcall=0
    tdtime=FUNCTION timer_start()                     ;touchdown time
    RETURN

;-----------------------------------------------------------------------------
; Altitude Alert
;-----------------------------------------------------------------------------
alt_alert:
    IF AUTOPILOT_ALTITUDE>0
        altdev=MTOF(AUTOPILOT_ALTITUDE-PLANE_ALT)                       ;[m]
;        altdev=AUTOPILOT_ALTITUDE-PLANE_ALT                             ;[-a]
        IF alertalt=AUTOPILOT_ALTITUDE       
            IF INRANGE(altdev,-900,900)                  
                IF INRANGE(altdev,-200,200)        
                    altalert=0
                ELSEIF altalert=0                     ;deviation 200-900ft
                    GOSUB alt_arm                     ;200ft exit alert or
                    GOTO play_altalert                ;900ft entry alert
                ENDIF
            ELSEIF altalert>0 
                IF altalert=2                         ;after 900ft entry:
                    altalert=1                        ;    900ft exit alert
                    GOSUB play_altalert               ;(avoid double alert
                ENDIF                                 ; 200ft + 900ft exit)
                altalert=0                            
            ENDIF                                     
        ELSEIF alertalt=-AUTOPILOT_ALTITUDE 
            alertalt=-alertalt
            IF INRANGE(altdev,-900,900)               ;within 900ft
                GOTO alt_arm                          ;no alert
            ELSE
                altalert=0
            ENDIF
        ELSE
            alertalt=-AUTOPILOT_ALTITUDE              ;delay
        ENDIF
    ENDIF
    RETURN

alt_arm:
    IF altdev/VELOCITY_Y<0                            ;exit from 200ft
        altalert=1                                           
    ELSE                                              ;entry within 900ft:
        altalert=2                                    ;      arm exit warn
    ENDIF
    RETURN

play_altalert:
    IF FLAPS_LEFT_POSITION<25000 AND NOT (VOR_1_GS_AVAIL AND INRANGE(VOR_1_GS_DEVIATION,-.2,.2))  ;[m]
;    IF FLAPS_LEFT_POSITION<.76 AND NOT (VOR_1_GS_AVAIL AND INRANGE(VOR_1_GS_DEVIATION,-.2,.2))    ;[-a]
;        PLAY ""              ;{delete for module}
        IF altalert=1
            PLAY "-altalrt"
            WAIT .7
        ELSE
            PLAY "-altentr"
            WAIT 1
        ENDIF
    ENDIF
    RETURN

;-----------------------------------------------------------------------------
; AP disconnect and Trim
;-----------------------------------------------------------------------------
ap_disc:
    IF AUTOPILOT_MASTER=0 
        APL_SOUND=1
;        PLAY ""              ;{delete for module}
        PLAY "-apdisco"
        WAIT .7
        trim=ELEVATOR_TRIM_POSITION
    ENDIF
    RETURN

trim:
    IF trim!=ELEVATOR_TRIM_POSITION AND (AUTOPILOT_MASTER=0 OR (NOT INRANGE(trim-ELEVATOR_TRIM_POSITION,-1677,1677) AND trimsound>1)) AND trimsound>0  ;[m]
;    IF trim!=ELEVATOR_TRIM_POSITION AND (AUTOPILOT_MASTER=0 OR (NOT INRANGE(trim-ELEVATOR_TRIM_POSITION,-.1,.1) AND trimsound>1)) AND trimsound>0      ;[-a]
        PLAY "-trim"
        WAIT .4
        trim=ELEVATOR_TRIM_POSITION
    ENDIF
    RETURN

;-----------------------------------------------------------------------------
; Settings
;-----------------------------------------------------------------------------
set_dh:
    PRINT_TIME 3
    PRINT
    PLAY "-ss1"
    IF dh=0
        PRINT "DH 20"
        dh=20
    ELSEIF dh=20 
        PRINT "DH 50"
        dh=50
    ELSEIF dh=50 
        PRINT "DH 100"
        dh=100
    ELSEIF dh=100 
        PRINT "DH 200"
        dh=200
    ELSEIF dh=200 
        PRINT "DH 300"
        dh=300
    ELSEIF dh=300 
        PRINT "DH 400"
        dh=400
    ELSEIF dh=400 
        PRINT "DH 500"
        dh=500
    ELSE
        PRINT "DH OFF"
        dh=0
    ENDIF
    RETURN

set_gpws:
    PRINT_TIME 3
    PRINT
    PLAY "-ss1"
    IF gpws=0
        PRINT "GPWS ON"
        gpws=5
        lastcall=0
        sinkalt=0
    ELSEIF gpws=5
        PRINT "GPWS g/s-inhb"
        gpws=4
    ELSEIF gpws=4
        PRINT "GPWS flap-ovrd"
        gpws=3
    ELSEIF gpws=3
        PRINT "GPWS gear-ovrd"
        gpws=2
    ELSEIF gpws=2
        PRINT "GPWS alt-only"
        gpws=1
    ELSE
        PRINT "GPWS OFF"
        lastcall=0
        gpws=0
    ENDIF
    RETURN

set_bankangle:
    PRINT_TIME 3
    PRINT
    PLAY "-ss1"
    IF bankangle=0
        PRINT "BANK ANGLE 65"
        bankangle=65
    ELSEIF bankangle=65
        PRINT "BANK ANGLE 50"
        bankangle=50
    ELSEIF bankangle=50
        PRINT "BANK ANGLE 35"
        bankangle=35
    ELSE
        PRINT "BANK ANGLE OFF"
        bankangle=0
    ENDIF
    RETURN

set_trimsound:
    PRINT_TIME 3
    PRINT
    PLAY "-ss1"
    IF trimsound=0
        PRINT "TRIM SOUND ON"
        trimsound=2
    ELSEIF trimsound=2
        PRINT "TRIM SOUND manual-only"
        trimsound=1
    ELSE
        PRINT "TRIM SOUND OFF"
        trimsound=0
    ENDIF
    RETURN
